#include <cppunit/Exception.h>
#include <cppunit/SourceLine.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestSuite.h>
#include <cppunit/TextOutputter.h>
#include <cppunit/TextTestProgressListener.h>
#include <cppunit/TestResultCollector.h>


CPPUNIT_NS_BEGIN


TextOutputter::TextOutputter( TestResultCollector *result,
                              OStream &stream )
    : m_result( result )
    , m_stream( stream )
{
}


TextOutputter::~TextOutputter()
{
}


void 
TextOutputter::write() 
{
  printHeader();
  m_stream << "\n";
  printFailures();
  m_stream << "\n";
}


void 
TextOutputter::printFailures()
{
  TestResultCollector::TestFailures::const_iterator itFailure = m_result->failures().begin();
  int failureNumber = 1;
  while ( itFailure != m_result->failures().end() ) 
  {
    m_stream  <<  "\n";
    printFailure( *itFailure++, failureNumber++ );
  }
}


void 
TextOutputter::printFailure( TestFailure *failure,
                             int failureNumber )
{
  printFailureListMark( failureNumber );
  m_stream << ' ';
  printFailureTestName( failure );
  m_stream << ' ';
  printFailureType( failure );
  if (failure->failedTest()->getType() == Test::IGNORED) {
      m_stream << " (ignored)";
  }
  m_stream << ' ';
  printFailureLocation( failure->sourceLine() );
  m_stream << "\n";
  printFailureDetail( failure->thrownException() );
  m_stream << "\n";
}


void 
TextOutputter::printFailureListMark( int failureNumber )
{
  m_stream << failureNumber << ")";
}


void 
TextOutputter::printFailureTestName( TestFailure *failure )
{
  m_stream << "test: " << failure->failedTestName();
}


void 
TextOutputter::printFailureType( TestFailure *failure )
{
  m_stream << "("
           << (failure->isError() ? "E" : "F")
           << ")";
}


void 
TextOutputter::printFailureLocation( SourceLine sourceLine )
{
  if ( !sourceLine.isValid() )
    return;

  m_stream << "line: " << sourceLine.lineNumber()
           << ' ' << sourceLine.fileName();
}


void 
TextOutputter::printFailureDetail( Exception *thrownException )
{
  m_stream  <<  thrownException->message().shortDescription()  <<  "\n";
  m_stream  <<  thrownException->message().details();
}


void 
TextOutputter::printHeader()
{
  if ( m_result->wasSuccessful() )
  {
    unsigned long ignored = TextTestProgressListener::ignoredTests;
    unsigned long disabled = TestSuite::disabledTests;
    m_stream << "\nOK (" << (m_result->runTests() - ignored) << " tests)";
    if (ignored > 0) {
        m_stream << " (Ignored " << ignored << " failing test"
                 << (ignored > 1 ? "s" : "") << ")";
    }
    if (disabled > 0) {
        m_stream << " (" << disabled << " test"
                 << (disabled > 1 ? "s" : "") << " disabled)";
    }
    m_stream << "\n";
  } else {
    m_stream << "\n";
    printFailureWarning();
    printStatistics();
  }
}


void 
TextOutputter::printFailureWarning()
{
  m_stream  << "!!! FAILURES !!!\n";
}


void 
TextOutputter::printStatistics()
{
unsigned long ignored = TextTestProgressListener::ignoredTests;
unsigned long disabled = TestSuite::disabledTests;
  m_stream  << "Test Results:\n";

  m_stream  <<  "Run:  "  <<  m_result->runTests()
            <<  "   Failures: "  <<  (m_result->testFailures() - ignored)
            <<  "   Ignored Failures: " << ignored
            <<  "   Errors: "  <<  m_result->testErrors()
            <<  "\n";
}


CPPUNIT_NS_END

